<?php
namespace Jazzaaf\xenAntiVirus\Cron;

use XF\Entity\AttachmentData;

class AttachmentCheck
{
	public static function runAttachmentCheck()
	{

		// get options
		$options = \XF::options();	
		$db = \XF::db();
		$em = \XF::app()->em();
		
		$virustotal_api_key = $options->jz_xenAntiVirus_api_key;
		$receivers = $options->jz_xenAntiVirus_alert_receivers;

		// make sure we have an API key
		if (!$virustotal_api_key)
		{
			\XF::app()->error()->logError('[xenAntiVirus] ' . \XF::phrase('jz_xenAntiVirus_no_api_key'));
			return;
		}		
		
		// get attachments that are not clean, and associated with a post. 
		// unassociated attachments will not be scanned
        $finder = \XF::finder('Jazzaaf\xenAntiVirus:AttachmentCheck');
        $attachmentFinder = $finder
 			->with(['Attachment', 'Data'], true)
            ->where('check_status', '!=', 'clean')
            ->where('check_status', '!=', 'infected')
            ->where('Attachment.unassociated', 0)
            ->order('check_id', 'ASC')
            ->fetch();

	
		foreach ($attachmentFinder as $k => $v)
		{
		
			// get dataId
			$data_id = $v['data_id'];
			
			// get fileHash
			$file_hash = $v['file_hash'];

			$attachData = $em->find('XF:AttachmentData', $data_id);
			$abstractedPath = $attachData->getAbstractedDataPath();

			$file_to_scan = \XF\Util\File::copyAbstractedPathToTempFile($attachData->getAbstractedDataPath(), false);

	
			if (!$file_hash) {
			// new file, not submitted before. Submit.
	
				$file_size = filesize($file_to_scan);
				$file_size_mb = $file_size/1024/1024;
	
				// default url where to post files
				$post_url = 'https://www.virustotal.com/vtapi/v2/file/scan';
	
				// get a special URL for uploading files larger than 32MB (up to 200MB)
				if($file_size_mb >= 32) {
				
					$api_reply = @file_get_contents('https://www.virustotal.com/vtapi/v2/file/scan/upload_url?apikey='.$virustotal_api_key);
					$api_reply_array = json_decode($api_reply, true);
					
					if(isset($api_reply_array['upload_url']) and $api_reply_array['upload_url']!='') {
						$post_url = $api_reply_array['upload_url'];
					}
				}
	
				// send a file for checking
				$cfile = curl_file_create($file_to_scan);
				$post = array('apikey' => $virustotal_api_key, 'file' => $cfile);
				$ch = curl_init();
				curl_setopt($ch, CURLOPT_URL, $post_url);
				curl_setopt($ch, CURLOPT_POST, True);
				curl_setopt($ch, CURLOPT_VERBOSE, 1); // remove this if your not debugging
				curl_setopt($ch, CURLOPT_ENCODING, 'gzip,deflate'); // please compress data
				curl_setopt($ch, CURLOPT_USERAGENT, "gzip, My php curl client");
				curl_setopt($ch, CURLOPT_RETURNTRANSFER, True);
				curl_setopt($ch, CURLOPT_POSTFIELDS, $post);

				$api_reply = curl_exec($ch);
				curl_close($ch);
	
				$api_reply_array = json_decode($api_reply, true);

				//var_dump($api_reply_array) . "<hr>";
	
				if($api_reply_array['response_code']==1) {
		
					$db->query("UPDATE xf_jz_antivirus_check SET last_message = '" . $api_reply_array['verbose_msg'] . "', last_check_date = " . time() . ", check_status = 'pending', file_hash = '" . $api_reply_array['sha256'] . "', scan_url = '" . $api_reply_array['permalink'] . "' WHERE check_id = " . $v['check_id']);
	
				}
	
			} else {
			// file has been submited already and has a hash value, check status.
	
				$report_url = 'https://www.virustotal.com/vtapi/v2/file/report?apikey='.$virustotal_api_key."&resource=".$file_hash;

				$api_reply = file_get_contents($report_url);
		
				// convert the json reply to an array of variables
				$api_reply_array = json_decode($api_reply, true);
				
				//var_dump($api_reply_array) . "<hr>";				

				// your resource is queued for analysis
				if($api_reply_array['response_code']==-2) {

					$db->query("UPDATE xf_jz_antivirus_check SET last_message = '" . $api_reply_array['verbose_msg'] . "', last_check_date = " . time() . ", check_status = 'pending' WHERE check_id = " . $v['check_id']);
		
				}
		
				// reply is OK (it contains an antivirus report)
				// use the variables from $api_reply_array to process the antivirus data
				if($api_reply_array['response_code']==1) {
	
					$positives_count = $api_reply_array['positives'];
		
				if ($positives_count > 0) {
					$check_status = "infected";
				
				// send alert
					if ($receivers) {
					
						foreach ($receivers as $receiver) {
						
							$receiver_id = $receiver;
							$user = \XF::app()->find('XF:User', $receiver_id);
							$alertRepo = \XF::app()->repository('XF:UserAlert');
							$alertRepo->alert($user, $user->user_id, $user->username, 'user', $user->user_id, 'jz_xenAntiVirus');
						
						}
						
					}
	
				} else {
					$check_status = "clean";
					
				}
	
					$db->query("UPDATE xf_jz_antivirus_check SET last_message = '" . $api_reply_array['verbose_msg'] . "', last_check_date = " . time() . ", check_status = '" . $check_status . "', file_hash = '" . $api_reply_array['sha256'] . "', scan_url = '" . $api_reply_array['permalink'] . "' WHERE check_id = " . $v['check_id']);

				}
				
			}
		

		}
		
	}

}